from dataclasses import dataclass

import sqlite3
import sys


IMDBFILE = "title.basics.tsv"
DBFILE = "db/im.db"
# imdb_id, media_type, original_media_type, primary_title, original_title, is_adult, start_year, end_year, runtime, genres
INSERT = "insert into imdb_records values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"

MEDIA_TYPE_MAPPING = {
                        "short" : "Short", # using "Kurzfilm" leads to issues when matching type 
                        "movie" : "Film", 
                        "tvMovie" : "Film", 
                        "tvShort" : "Short", # using "Kurzfilm" leads to issues when matching type 
                        "tvSeries" : "Serie", 
                        "tvMiniSeries" : "Serie", 
                        "tvSpecial" : "Serie;Film;Doku;Short", 
                        "video" : "Serie;Film;Doku;Short", 
                        "tvPilot" : "Serie;Film", 
                        # filtered:
                        # "videoGame" : "Spiel", 
                        # "tvEpisode" : "Serie", 
                     }


values = []


@dataclass
class ImdbRecord():
    imdb_id : str
    media_type : str
    original_media_type : str
    primary_title : str
    original_title : str
    is_adult : bool
    start_year : int
    end_year : int
    runtime : int
    genres : str

    def get_values(self) -> tuple:
        return (self.imdb_id, self.media_type, self.original_media_type, \
                self.primary_title, self.original_title, self.is_adult, \
                self.start_year, self.end_year, self.runtime, self.genres)


def bulk_import() -> None:
    try:
        conn = sqlite3.connect(DBFILE)
    except:
        sys.exit()

    with conn:
        curs = conn.cursor()
        curs.executemany(INSERT, values)
        conn.commit()
        curs.close()
        values.clear()


def read_file() -> str:
    with open(IMDBFILE) as imdb_file:
        lines = imdb_file.readlines()
    return lines


def create_records(lines: list) -> None:
    for line in lines:
        if "tvEpisode" in line or "videoGame" in line:
            continue
        line = line.strip("\n")
        line = line.replace('\\N', "")
        fields = line.split("\t")
        rec = ImdbRecord(imdb_id = fields[0], \
                         media_type = MEDIA_TYPE_MAPPING[fields[1]], \
                         original_media_type = fields[1], \
                         primary_title = fields[2], original_title = fields[3], \
                         is_adult = fields[4], \
                         start_year = fields[5], end_year = fields[6], \
                         runtime = fields[7], genres = fields[8])
        values.append(rec.get_values())


chunk_size = 10000 # choose depending on your system

start_line = 1 # skip header
end_line = chunk_size


lines = read_file()

while end_line < len(lines):
    create_records(lines[start_line:end_line])
    bulk_import()
    start_line = end_line
    end_line = end_line + chunk_size if len(lines) > end_line + chunk_size else len(lines)


